@@ -75,6 +75,7 @@ ISOTREE_EXPORTED int fit_iforest(IsoForest *model_outputs, ExtIsoForest *model_o
                 double output_depths[], bool standardize_depth,
                 real_t col_weights[], bool weigh_by_kurt,
                 double prob_pick_by_gain_pl, double prob_pick_by_gain_avg,
+                double prob_pick_by_full_gain, double prob_pick_by_dens,
                 double prob_pick_col_by_range, double prob_pick_col_by_var,
                 double prob_pick_col_by_kurt,
                 double min_gain, MissingAction missing_action,
@@ -98,6 +99,7 @@ ISOTREE_EXPORTED int fit_iforest(IsoForest *model_outputs, ExtIsoForest *model_o
                 output_depths, standardize_depth,
                 col_weights, weigh_by_kurt,
                 prob_pick_by_gain_pl, prob_pick_by_gain_avg,
+                prob_pick_by_full_gain, prob_pick_by_dens,
                 prob_pick_col_by_range, prob_pick_col_by_var,
                 prob_pick_col_by_kurt,
                 min_gain, missing_action,
@@ -117,6 +119,7 @@ ISOTREE_EXPORTED int add_tree(IsoForest *model_outputs, ExtIsoForest *model_outp
              bool   fast_bratio,
              real_t col_weights[], bool weigh_by_kurt,
              double prob_pick_by_gain_pl, double prob_pick_by_gain_avg,
+             double prob_pick_by_full_gain, double prob_pick_by_dens,
              double prob_pick_col_by_range, double prob_pick_col_by_var,
              double prob_pick_col_by_kurt,
              double min_gain, MissingAction missing_action,
@@ -141,6 +144,7 @@ ISOTREE_EXPORTED int add_tree(IsoForest *model_outputs, ExtIsoForest *model_outp
              fast_bratio,
              col_weights, weigh_by_kurt,
              prob_pick_by_gain_pl, prob_pick_by_gain_avg,
+             prob_pick_by_full_gain, prob_pick_by_dens,
              prob_pick_col_by_range, prob_pick_col_by_var,
              prob_pick_col_by_kurt,
              min_gain, missing_action,
